
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <fcntl.h>
#include <jni.h>
#include <errno.h>


int readStringFromFile(char* file, char* str, int maxsize){
	FILE* fh;
	int read;
	
	fh = fopen(file, "r");
	if(fh == NULL)
		return -1;
	
	read = fread(str, 1, maxsize, fh);
	
	str[read] = '\0';
	
	fclose(fh);
}

jstring
Java_de_incircuit_hw_adbjni_ADBJni_readStringFromFile( 	JNIEnv* env,
                                                				jobject thiz,
																jstring file,
																jint maxsize)
{
	const char *cfile = (*env)->GetStringUTFChars(env, file, 0);
	char buffer[maxsize];
	
	if(readStringFromFile((char*) cfile, buffer, maxsize) < 0)
		return NULL;
	
	return (*env)->NewStringUTF(env, buffer);
}

int writeStringToFile(char* file, char* str, char* err){
	FILE* fh;
	fh = fopen(file, "w");
	if(fh == NULL){
		sprintf(err, "Error: %s\n", strerror(errno));
		return -1;
	}
	fprintf(fh, str);
	fclose(fh);
}

jstring
Java_de_incircuit_hw_adbjni_ADBJni_writeStringToFile( 	JNIEnv* env,
                                                				jobject thiz,
																jstring file,
																jstring value)
{
	const char *cfile = (*env)->GetStringUTFChars(env, file, 0);
	const char *cstr = (*env)->GetStringUTFChars(env, value, 0);
	char err[256];
	err[0] = '\0';
	
	writeStringToFile((char*) cfile, (char*)  cstr, err);
	
	return (*env)->NewStringUTF(env, err);
}

int uart_open(char* file, int baud){
	int uart;
	struct termios t;
	
	memset(&t, 0, sizeof(t));
	
	uart = open(file, O_RDWR);
	if(uart < 0)
		return uart;
	
	cfmakeraw(&t);
	
	if(baud == 9600){
		cfsetispeed(&t, B9600);
		cfsetospeed(&t, B9600);
	} else if(baud == 19200){
		cfsetispeed(&t, B19200);
		cfsetospeed(&t, B19200);
	} else if(baud == 57600){
		cfsetispeed(&t, B57600);
		cfsetospeed(&t, B57600);
	} else if(baud == 115200){
		cfsetispeed(&t, B115200);
		cfsetospeed(&t, B115200);
	} else{
		close(uart);
		return -1;
	}
	
	t.c_cflag |= CS8;
	tcsetattr(uart, TCSANOW, &t);
	
	return uart;
}

jint
Java_de_incircuit_hw_adbjni_ADBJni_uart_open( 	JNIEnv* env,
                                                				jobject thiz,
																jstring file,
																jint baud)
{
	const char *cfile = (*env)->GetStringUTFChars(env, file, 0);
	
	return uart_open((char*) cfile, baud);
}


int uart_read(int uart, char* buffer, int size){
	return read(uart, buffer, size);
}

jint
Java_de_incircuit_hw_adbjni_ADBJni_uart_read( 	JNIEnv* env,
                                                				jobject thiz,
																jint uart,
																jcharArray buffer,
																jint size)
{
	char* buf = (*env)->GetCharArrayElements(env, buffer, NULL);
	
	int ret = uart_read(uart, buf, size);
	
	(*env)->ReleaseCharArrayElements(env, buffer, buf, 0);
	
	return ret;
}

void uart_write(int uart, char* buffer, int size) {
	write(uart, buffer, size);
}

void
Java_de_incircuit_hw_adbjni_ADBJni_uart_write( 	JNIEnv* env,
                                                				jobject thiz,
																jint uart,
																jcharArray buffer,
																jint size)
{
	char* buf = (*env)->GetCharArrayElements(env, buffer, NULL);
	
	uart_write(uart, buf, size);
}

void uart_close(int uart){
	close(uart);
}

void 
Java_de_incircuit_hw_adbjni_ADBJni_uart_close( 	JNIEnv* env,
                                                				jobject thiz,
																jint uart)
{
	uart_close(uart);
}



