package de.incircuit.demo.gpio;

import de.incircuit.hw.ADC;
import de.incircuit.hw.ADC.CALIB;
import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ArrayAdapter;
import android.widget.Spinner;
import android.widget.TextView;

public class adcActivity extends Activity {
	private static final String TAG = "adcActivity.java";

	ADC adc[] = new ADC[2];
	String types[] = { "current (4-20mA)", "voltage (0-10V)" };

	private final int ADC_TIMER_TIMEOUT = 1000;
	private Handler mHandler = new Handler();
	private Runnable adcTimerTask = new Runnable() {
		// @Override
		public void run() {
			refresh_status();
			mHandler.postDelayed(this, ADC_TIMER_TIMEOUT);
		}

	};

	public class AddItemSelected implements OnItemSelectedListener {
		private int idx;

		public AddItemSelected(int idx) {
			this.idx = idx;
		}

		public void onItemSelected(AdapterView<?> parent, View view, int pos,
				long id) {
			adc[idx].set_type(types[pos]);
			Log.d(TAG, "set new type=" + types[pos]);
		}

		public void onNothingSelected(AdapterView<?> parent) {

		}
	};

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.adc);

		for (int i = 0; i < 2; i++)
			adc[i] = new ADC(i);

		Spinner spinner = (Spinner) findViewById(R.id.SpinnerADCtype0);
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_spinner_item, types);
		adapter.setDropDownViewResource(android.R.layout.simple_dropdown_item_1line);
		spinner.setAdapter(adapter);
		if (adc[0].get_type().contains(types[0])) {
			spinner.setSelection(0);
		} else {
			spinner.setSelection(1);
		}
		spinner.setOnItemSelectedListener(new AddItemSelected(0));

		spinner = (Spinner) findViewById(R.id.SpinnerADCtype1);
		adapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_spinner_item, types);
		adapter.setDropDownViewResource(android.R.layout.simple_dropdown_item_1line);
		spinner.setAdapter(adapter);
		if (adc[1].get_type().contains(types[0])) {
			spinner.setSelection(0);
		} else {
			spinner.setSelection(1);
		}
		spinner.setOnItemSelectedListener(new AddItemSelected(1));

		TextView tv = (TextView) findViewById(R.id.ADCvalue0);
		tv.setText(String.format("%d", adc[0].get_value()));

		tv = (TextView) findViewById(R.id.ADCvalue1);
		tv.setText(String.format("%d", adc[1].get_value()));

		CALIB calib = adc[0].get_calib();

		tv = (TextView) findViewById(R.id.EditTextVMax0);
		tv.setText(String.format("%d", calib.v_max));
		tv = (TextView) findViewById(R.id.EditTextVMin0);
		tv.setText(String.format("%d", calib.v_min));
		tv = (TextView) findViewById(R.id.EditTextCMax0);
		tv.setText(String.format("%d", calib.c_max));
		tv = (TextView) findViewById(R.id.EditTextCMin0);
		tv.setText(String.format("%d", calib.c_min));

		calib = adc[1].get_calib();

		tv = (TextView) findViewById(R.id.EditTextVMax1);
		tv.setText(String.format("%d", calib.v_max));
		tv = (TextView) findViewById(R.id.EditTextVMin1);
		tv.setText(String.format("%d", calib.v_min));
		tv = (TextView) findViewById(R.id.EditTextCMax1);
		tv.setText(String.format("%d", calib.c_max));
		tv = (TextView) findViewById(R.id.EditTextCMin1);
		tv.setText(String.format("%d", calib.c_min));

	}

	private void refresh_status(){
		TextView tv = (TextView) findViewById(R.id.ADCvalue0);
		tv.setText(String.format("%d", adc[0].get_value()));

		tv = (TextView) findViewById(R.id.ADCvalue1);
		tv.setText(String.format("%d", adc[1].get_value()));
	}
	
	public void onPause() {
		mHandler.removeCallbacks(adcTimerTask);
		super.onPause();
	}

	public void onResume() {
		super.onResume();
		mHandler.removeCallbacks(adcTimerTask);
		mHandler.postDelayed(adcTimerTask, 1);
	}

	public void onClickWriteCalib0(View view) {
		CALIB calib = adc[0].get_calib();

		try {
			TextView tv = (TextView) findViewById(R.id.EditTextVMax0);
			calib.v_max = Integer.parseInt(tv.getText().toString());
			tv = (TextView) findViewById(R.id.EditTextVMin0);
			calib.v_min = Integer.parseInt(tv.getText().toString());
			tv = (TextView) findViewById(R.id.EditTextCMax0);
			calib.c_max = Integer.parseInt(tv.getText().toString());
			tv = (TextView) findViewById(R.id.EditTextCMin0);
			calib.c_min = Integer.parseInt(tv.getText().toString());
		} catch (NumberFormatException e) {
			return;
		}

		adc[0].set_calib(calib);

		calib = adc[0].get_calib();

		TextView tv = (TextView) findViewById(R.id.EditTextVMax0);
		tv.setText(String.format("%d", calib.v_max));
		tv = (TextView) findViewById(R.id.EditTextVMin0);
		tv.setText(String.format("%d", calib.v_min));
		tv = (TextView) findViewById(R.id.EditTextCMax0);
		tv.setText(String.format("%d", calib.c_max));
		tv = (TextView) findViewById(R.id.EditTextCMin0);
		tv.setText(String.format("%d", calib.c_min));
	}

	public void onClickWriteCalib1(View view) {
		CALIB calib = adc[1].get_calib();
		try {
			TextView tv = (TextView) findViewById(R.id.EditTextVMax1);
			calib.v_max = Integer.parseInt(tv.getText().toString());
			tv = (TextView) findViewById(R.id.EditTextVMin1);
			calib.v_min = Integer.parseInt(tv.getText().toString());
			tv = (TextView) findViewById(R.id.EditTextCMax1);
			calib.c_max = Integer.parseInt(tv.getText().toString());
			tv = (TextView) findViewById(R.id.EditTextCMin1);
			calib.c_min = Integer.parseInt(tv.getText().toString());
		} catch (NumberFormatException e) {
			return;
		}
		adc[1].set_calib(calib);

		calib = adc[1].get_calib();

		TextView tv = (TextView) findViewById(R.id.EditTextVMax1);
		tv.setText(String.format("%d", calib.v_max));
		tv = (TextView) findViewById(R.id.EditTextVMin1);
		tv.setText(String.format("%d", calib.v_min));
		tv = (TextView) findViewById(R.id.EditTextCMax1);
		tv.setText(String.format("%d", calib.c_max));
		tv = (TextView) findViewById(R.id.EditTextCMin1);
		tv.setText(String.format("%d", calib.c_min));

	}
}
