package de.incircuit.hw;

import android.util.Log;
import de.incircuit.hw.adbjni.ADBJni;

public class ADC {
	private static final String TAG = "ADC.java";
	private static final boolean DEBUG = false;
	private static final int timeout = 50;
	
	private ADBJni hw;
	private int num;

	private static final String ADC_DIR = "/sys/bus/spi/devices/spi0.0/adc";
	//private static final String ADC_DIR = "/data/adc/adc";
	
	private static final String ADC_RAW = "raw";
	private static final String ADC_VALUE = "value";
	private static final String ADC_TYPE = "type";
	private static final String ADC_CALIB_V_MAX = "calibVmax";
	private static final String ADC_CALIB_V_MIN = "calibVmin";
	private static final String ADC_CALIB_C_MAX = "calibCmax";
	private static final String ADC_CALIB_C_MIN = "calibCmin";
	
	private boolean calibrating = false;

	public class CALIB {
		public int v_max;
		public int v_min;
		public int c_max;
		public int c_min;

		public String toString() {
			return "{vmax=" + v_max + " vmin=" + v_min + " cmax=" + c_max
					+ " cmin=" + c_min + "}";
		}
	};

	public ADC(int num) {
		if(DEBUG)
			Log.d(TAG, "ADC(num="+num+")");
		
		hw = new ADBJni();
		this.num = num;
	}

	public int get_value() {
		if(DEBUG)
			Log.d(TAG, "get_value()");
		if(calibrating) return 0;
		return hw.readIntegerFromFile(ADC_DIR + num + ADC_VALUE);
	}

	public int get_raw() {
		if(DEBUG)
			Log.d(TAG, "get_raw()");
		if(calibrating) return 0;
		return hw.readIntegerFromFile(ADC_DIR + num + ADC_RAW);
	}

	public String get_type() {
		String value = hw.readStringFromFile(ADC_DIR + num + ADC_TYPE, 32);

		if(DEBUG)
			Log.d(TAG, "get_type() str=" + value);
		
		if (value == null)
			return "";
		if (value.contains("[voltage]"))
			return "voltage";
		else
			return "current";
	}

	public void set_type(String type) {
		if(DEBUG)
			Log.d(TAG, "set_type(type=" + type+")");
		hw.writeStringToFile(ADC_DIR + num + ADC_TYPE, type);
		try {
			Thread.sleep(50);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	public void set_calib(CALIB calib) {
		calibrating = true;
		String type = get_type();
		if(DEBUG)
			Log.d(TAG, "set_calib(calib=" + calib.toString()+")");
		
		hw.writeIntegerToFile(ADC_DIR + num + ADC_CALIB_V_MAX, calib.v_max);
		try {
			Thread.sleep(timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		hw.writeIntegerToFile(ADC_DIR + num + ADC_CALIB_V_MIN, calib.v_min);
		try {
			Thread.sleep(timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		hw.writeIntegerToFile(ADC_DIR + num + ADC_CALIB_C_MAX, calib.c_max);
		try {
			Thread.sleep(timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		hw.writeIntegerToFile(ADC_DIR + num + ADC_CALIB_C_MIN, calib.c_min);
		try {
			Thread.sleep(2*timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		Log.d(TAG, "resetting type to "+type);
		set_type(type);
		calibrating = false;
	}

	public CALIB get_calib() {
		calibrating = true;
		String type = get_type();
		CALIB calib = new CALIB();

		calib.v_max = hw.readIntegerFromFile(ADC_DIR + num + ADC_CALIB_V_MAX);
		calib.v_min = hw.readIntegerFromFile(ADC_DIR + num + ADC_CALIB_V_MIN);
		calib.c_max = hw.readIntegerFromFile(ADC_DIR + num + ADC_CALIB_C_MAX);
		calib.c_min = hw.readIntegerFromFile(ADC_DIR + num + ADC_CALIB_C_MIN);
		
		if(DEBUG)
			Log.d(TAG, "get_calib() calib=" + calib.toString());
		try {
			Thread.sleep(2*timeout);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		Log.d(TAG, "resetting type to "+type);
		set_type(type);
		calibrating = false;
		return calib;
	}

}
